#ifndef _CUBE_H
#define _CUBE_H

#include <stdint.h>
#include "common.h"
#include "uip/uip.h"

/*
Hardwarekonfiguration:
----------------------

Registerreihenfolge (von SER gesehen):
(Spalten/Ebenen nochmal prüfen)
1: Spalten
2: Ebenen
3: Zeilen

PC4 -> SER \
PC5 -> SCK   \ 8Bit-Schieberegister 
PC6 -> RCK   / (74xx595x; G auf L)
PC7 -> SCL /
*/
#define PORT_SR PORTC
#define PIN_SER	PC4
#define PIN_SCK PC5
#define PIN_RCK PC6
#define PIN_SCL PC7


/* TCP/IP-Port */
#define CUBE_PORT 12345

/* Cube-Dimension */
#define CUBE_X 8
#define CUBE_Y 8
#define CUBE_Z 8
#define CUBE_XYZ (CUBE_X * CUBE_Y * CUBE_Z)

/* Timer1-Werte berechnen */
#define CUBE_REFRESH    60						/* Bildwiederholfrequenz in Hz */
#define T1_PRESCALE     1024						/* Vorteiler des Timers        */
#define T1_MAX_COUNTER  65535						/* Maximalwert des Timers      */

#ifdef SINGLE_LED
#define T1_FREQ         (CUBE_REFRESH * CUBE_Y * CUBE_Z * CUBE_X)	/* Zeilenfrequenz              */
#else
#define T1_FREQ         (CUBE_REFRESH * CUBE_Y * CUBE_Z)		/* Zeilenfrequenz              */
#endif

#define T1_PRELOAD      (T1_MAX_COUNTER-(F_CPU/T1_PRESCALE/T1_FREQ))	/* resultierender Preloadwert  */


/* Befehlswoerter */
#define CMD_SET_LED       "SET_LED"
#define CMD_GET_XYZ       "GET_XYZ"
#define CMD_VERSION       "VERSION"
#define CMD_SET_PRELOAD   "SET_PRELOAD"
#define CMD_HELP          "HELP"


/* Zeitpunkt der Uebersetzung (wird im Makefile gesetzt) */
#ifndef BUILD_DATE
#define BUILD_DATE "unknown\n"
#endif
static const char build_date[] = BUILD_DATE;

/* Antwort auf Befehl GET_XYZ */
#define TOSTR(val) STR(val)
#define STR(val) #val
#define OUT_SET_XYZ "set_xyz " TOSTR(CUBE_X) " " TOSTR(CUBE_Y) " " TOSTR(CUBE_Z) "\n"
static const char out_set_xyz[] = OUT_SET_XYZ;

/* Antwort auf Befehle SET_LED */
#define OUT_UPDATE_OK "update_ok\n"
static const char out_update_ok[] = OUT_UPDATE_OK;

/* Prototypen */
void SplitCmdArg(char * line, char ** cmd, char ** args);
void cube_init(void);
void cube_main(void);

/* Typ-Definitionen */
struct cube_cmd_state {
	enum {CMD_SET_LED_RECEIVE, NEW_CMD} state;
	uint16_t  arg_pos;		/* naechste zu uebernehmende Position bei set_led */
	char     *textptr;
	uint16_t  textlen;
};


#endif
